@@ -13,6 +13,8 @@ class MeinheldWorker(Worker):
 
     def run(self):
         fd = self.socket.fileno()
+        server.set_keepalive(self.cfg.keepalive)
+        server.set_picoev_max_fd(self.cfg.worker_connections)
         server.set_watchdog(self.watchdog)
         server.set_listen_socket(fd)
         server.run(self.wsgi)
