@@ -617,6 +617,10 @@ execute_parse(client_t *cli, const char *data, size_t len)
     size_t ret = http_parser_execute(cli->http, &settings, data, len);
     //check new protocol
     cli->upgrade = cli->http->upgrade;
+    
+    cli->http_major = cli->http->http_major;
+    cli->http_minor = cli->http->http_minor;
+
     return ret;
 }
 
