@@ -50,7 +50,9 @@ static int backlog = 1024 * 4; // backlog size
 static int max_fd = 1024 * 4;  // picoev max_fd
 
 // greenlet hub switch value
-PyObject *hub_switch_value;
+PyObject* hub_switch_value;
+PyObject* current_client;
+PyObject* timeout_error;
 
 
 static void
@@ -104,7 +106,15 @@ clean_cli(client_t *client)
     Py_CLEAR(client->http_status);
     Py_CLEAR(client->headers);
     Py_CLEAR(client->response_iter);
+    
     Py_CLEAR(client->response);
+#ifdef DEBUG
+    printf("close environ %p \n", client->environ);
+#endif
+    
+    // force clear
+    PyDict_Clear(client->environ);
+    
     Py_CLEAR(client->environ);
     if(client->body_type == BODY_TYPE_TMPFILE){
         if(client->body){
@@ -129,11 +139,15 @@ close_conn(client_t *cli, picoev_loop* loop)
     }
 
     picoev_del(loop, cli->fd);
+
+#ifdef DEBUG
+    printf("picoev_del client:%p fd:%d \n", cli, cli->fd);
+#endif
     clean_cli(cli);
     if(!cli->keep_alive){
         close(cli->fd);
 #ifdef DEBUG
-        printf("close fd %d \n", cli->fd);
+        printf("close client:%p fd:%d \n", cli, cli->fd);
 #endif
     }else{
         disable_cork(cli);
@@ -159,6 +173,7 @@ process_resume_wsgi_app(ClientObject *pyclient)
     old_client = start_response->cli;
     start_response->cli = client;
     
+    current_client = (PyObject *)pyclient;
     if(PyErr_Occurred()){
         PyErr_Fetch(&err_type, &err_val, &err_tb);
         PyErr_Clear();
@@ -208,6 +223,10 @@ process_wsgi_app(client_t *cli)
     }
     args = Py_BuildValue("(OO)", cli->environ, start);
 
+    current_client = PyDict_GetItemString(cli->environ, "meinheld.client");
+#ifdef DEBUG
+    printf("start environ %p \n", cli->environ);
+#endif
     res = PyObject_CallObject(wsgi_app, args);
     Py_DECREF(args);
     
@@ -240,6 +259,8 @@ inline void
 switch_wsgi_app(picoev_loop* loop, int fd, PyObject *obj)
 {
     ClientObject *pyclient = (ClientObject *)obj;
+    
+    //clear event
     picoev_del(loop, fd);
     // resume
     resume_wsgi_app(pyclient, loop);
@@ -251,9 +272,61 @@ resume_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
     ClientObject *pyclient = (ClientObject *)(cb_arg);
     client_t *client = pyclient->client;
+    if ((events & PICOEV_TIMEOUT) != 0) {
+        PyErr_SetString(timeout_error, "timeout");
+        set_so_keepalive(client->fd, 0);
+        
+    }else if ((events & PICOEV_WRITE) != 0) {
+        //None
+    }
     switch_wsgi_app(loop, client->fd, (PyObject *)pyclient); 
 }
 
+static void
+timeout_error_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
+{
+    ClientObject *pyclient = (ClientObject *)(cb_arg);
+    client_t *client = pyclient->client;
+    
+    if ((events & PICOEV_TIMEOUT) != 0) {
+#ifdef DEBUG
+        printf("timeout_error_callback pyclient:%p client:%p fd:%d \n", pyclient, pyclient->client, pyclient->client->fd);
+#endif
+        pyclient->suspended = 0;
+        pyclient->resumed = 1;
+        PyErr_SetString(timeout_error, "timeout");
+        set_so_keepalive(client->fd, 0);
+        switch_wsgi_app(loop, client->fd, (PyObject *)pyclient); 
+    }
+}
+
+static void
+timeout_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
+{
+    ClientObject *pyclient = (ClientObject *)(cb_arg);
+    client_t *client = pyclient->client;
+    if ((events & PICOEV_TIMEOUT) != 0) {
+#ifdef DEBUG
+        printf("timeout_callback pyclient:%p client:%p fd:%d \n", pyclient, pyclient->client, pyclient->client->fd);
+#endif
+        //next intval 30sec
+        picoev_set_timeout(loop, client->fd, 30);
+        
+        // is_active ??
+        if(write(client->fd, "", 0) < 0){
+            //resume       
+            pyclient->suspended = 0;
+            pyclient->resumed = 1;
+            PyErr_SetFromErrno(PyExc_IOError);
+#ifdef DEBUG
+            printf("closed \n");
+#endif
+            set_so_keepalive(client->fd, 0);
+            switch_wsgi_app(loop, client->fd, (PyObject *)pyclient); 
+        }
+    }
+}
+
 
 static void
 w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
@@ -303,7 +376,6 @@ resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop)
             // suspend
             return;
         default:
-            pyclient->client = NULL;
             break;
     }
 
@@ -328,6 +400,8 @@ resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop)
 #ifdef DEBUG
             printf("set write callback %d \n", ret);
 #endif
+            //clear event
+            picoev_del(loop, client->fd);
             picoev_add(loop, client->fd, PICOEV_WRITE, WRITE_TIMEOUT_SECS, w_callback, (void *)client);
             return;
         default:
@@ -375,6 +449,8 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
 #ifdef DEBUG
             printf("set write callback %d \n", ret);
 #endif
+            //clear event
+            picoev_del(loop, client->fd);
             picoev_add(loop, client->fd, PICOEV_WRITE, WRITE_TIMEOUT_SECS, w_callback, (void *)client);
             return;
         default:
@@ -438,6 +514,9 @@ static void
 r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
     client_t *cli = ( client_t *)(cb_arg);
+    PyObject *body = NULL;
+    char *key = NULL;
+
     if ((events & PICOEV_TIMEOUT) != 0) {
 
 #ifdef DEBUG
@@ -530,8 +609,8 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                 if(parser_finish(cli) > 0){
                     if(cli->upgrade){
                         //WebSocket Key
-                        char *key = buf + nread + 1;
-                        PyObject *body = PyString_FromStringAndSize(key, r - nread);
+                        key = buf + nread + 1;
+                        body = PyString_FromStringAndSize(key, r - nread);
                         cli->body = body;
 
                     }
@@ -1039,9 +1118,11 @@ meinheld_suspend_client(PyObject *self, PyObject *args)
 {
     PyObject *temp;
     ClientObject *pyclient;
+    client_t *client;
     PyGreenlet *parent;
+    int timeout;
 
-    if (!PyArg_ParseTuple(args, "O:_suspend_client", &temp)){
+    if (!PyArg_ParseTuple(args, "O|i:_suspend_client", &temp, &timeout)){
         return NULL;
     }
     
@@ -1052,7 +1133,8 @@ meinheld_suspend_client(PyObject *self, PyObject *args)
     }
 
     pyclient = (ClientObject *)temp;
-    
+    client = pyclient->client;
+
     if(!pyclient->greenlet){
         PyErr_SetString(PyExc_ValueError, "greenlet is not set");
         return NULL;
@@ -1060,13 +1142,26 @@ meinheld_suspend_client(PyObject *self, PyObject *args)
 
     if(pyclient->resumed == 1){
         //call later
-        PyErr_SetString(PyExc_Exception, "not called resume ");
+        PyErr_SetString(PyExc_IOError, "not called resume");
         return NULL;
     }
     
-    if(pyclient->client && !(pyclient->suspended)){
+    if(client && !(pyclient->suspended)){
         pyclient->suspended = 1;
         parent = PyGreenlet_GET_PARENT(pyclient->greenlet);
+
+        set_so_keepalive(client->fd, 1);
+#ifdef DEBUG
+        printf("meinheld_suspend_client pyclient:%p client:%p fd:%d \n", pyclient, client, client->fd);
+        printf("meinheld_suspend_client active ? %d \n", picoev_is_active(main_loop, client->fd));
+#endif
+        //clear event
+        picoev_del(main_loop, client->fd);
+        if(timeout > 0){
+            picoev_add(main_loop, client->fd, PICOEV_TIMEOUT, timeout, timeout_error_callback, (void *)pyclient);
+        }else{
+            picoev_add(main_loop, client->fd, PICOEV_TIMEOUT, 300, timeout_callback, (void *)pyclient);
+        }
         return PyGreenlet_Switch(parent, hub_switch_value, NULL);
     }else{
         PyErr_SetString(PyExc_Exception, "already suspended");
@@ -1093,7 +1188,8 @@ meinheld_resume_client(PyObject *self, PyObject *args)
     }
 
     pyclient = (ClientObject *)temp;
-    
+    client = pyclient->client;
+
     if(!pyclient->greenlet){
         PyErr_SetString(PyExc_ValueError, "greenlet is not set");
         return NULL;
@@ -1101,20 +1197,26 @@ meinheld_resume_client(PyObject *self, PyObject *args)
 
     if(!pyclient->suspended){
         // not suspend
-        PyErr_SetString(PyExc_Exception, "not suspended");
+        PyErr_SetString(PyExc_IOError, "not suspended or dead ");
         return NULL;
     }
 
     if(pyclient->client && !pyclient->resumed){
+        set_so_keepalive(pyclient->client->fd, 0);
         pyclient->args = switch_args;
         Py_XINCREF(pyclient->args);
     
         pyclient->kwargs = switch_kwargs;
         Py_XINCREF(pyclient->kwargs);
-        client = pyclient->client;
 
         pyclient->suspended = 0;
         pyclient->resumed = 1;
+#ifdef DEBUG
+        printf("meinheld_resume_client pyclient:%p client:%p fd:%d \n", pyclient, pyclient->client, pyclient->client->fd);
+        printf("meinheld_resume_client active ? %d \n", picoev_is_active(main_loop, pyclient->client->fd));
+#endif
+        //clear event
+        picoev_del(main_loop, client->fd);
         picoev_add(main_loop, client->fd, PICOEV_WRITE, 0, resume_callback, (void *)pyclient);
     }else{
         PyErr_SetString(PyExc_Exception, "already resumed");
@@ -1127,15 +1229,14 @@ PyObject *
 meinheld_get_socket_fromfd(PyObject *self, PyObject *args)
 {
     int fd;
-    PyObject *temp;
-    if (!PyArg_ParseTuple(args, "iO:_get_socket_fromfd", &fd, &temp)){
+    if (!PyArg_ParseTuple(args, "i:_get_socket_fromfd", &fd)){
         return NULL;
     }
-    if(!CheckClientObject(temp)){
-        PyErr_SetString(PyExc_TypeError, "must be a client object");
+    if(!((ClientObject *)current_client)->greenlet){
+        PyErr_SetString(PyExc_ValueError, "greenlet is not set");
         return NULL;
     }
-    return NSocketObject_New(fd, (ClientObject *)temp);
+    return NSocketObject_New(fd, (ClientObject *)current_client);
 
 }
 
@@ -1174,10 +1275,31 @@ static PyMethodDef WsMethods[] = {
 PyMODINIT_FUNC
 initserver(void)
 {
-    (void)Py_InitModule("meinheld.server", WsMethods);
-    PyType_Ready(&FileWrapperType);
-    PyType_Ready(&ClientObjectType);
-    PyType_Ready(&NSocketObjectType);
+    PyObject *m;
+    m = Py_InitModule("meinheld.server", WsMethods);
+    if(m == NULL){
+        return;
+    }
+
+    if(PyType_Ready(&FileWrapperType) < 0){ 
+        return;
+    }
+
+    if(PyType_Ready(&ClientObjectType) < 0){
+        return;
+    }
+
+    if(PyType_Ready(&NSocketObjectType) < 0){
+        return;
+    }
+
+	timeout_error = PyErr_NewException("meinheld.server.timeout",
+					  PyExc_IOError, NULL);
+	if (timeout_error == NULL)
+		return;
+	Py_INCREF(timeout_error);
+	PyModule_AddObject(m, "timeout", timeout_error);
+
 #ifdef DEBUG
     printf("client size %d \n", sizeof(client_t));
 #endif
