@@ -25,7 +25,7 @@
 #include "http_parser.h"
 
 
-#define SERVER "meinheld/0.3.2"
+#define SERVER "meinheld/0.3.3"
 
 typedef enum {
     BODY_TYPE_NONE,
@@ -39,6 +39,9 @@ extern picoev_loop* main_loop; //main loop
 
 extern PyObject* hub_switch_value;
 
+extern PyObject* current_client;
+extern PyObject* timeout_error;
+
 inline void
 switch_wsgi_app(picoev_loop* loop, int fd, PyObject *obj);
 
